@@ -23,6 +23,7 @@
 
 #include <rtascollector.hpp>
 #include <platformcollector.hpp>
+#include <devicetreecollector.hpp>
 #include <libvpd-2/vpdretriever.hpp>
 #include <libvpd-2/component.hpp>
 #include <libvpd-2/dataitem.hpp>
@@ -39,6 +40,7 @@
 #define _GNU_SOURCE // for getopt_long
 #endif
 
+#include <dirent.h>
 #include <unistd.h>
 #include <getopt.h>
 #include <zlib.h>
@@ -50,6 +52,9 @@
 #include <iomanip>
 #include <limits.h>
 
+/* Firmware information device tree node on PowerNV system */
+#define FW_VERSION_DT_NODE DEVTREEPATH"/ibm,firmware-versions/"
+
 /* IPMI tool */
 #define CMD_IPMITOOL	"ipmitool"
 
@@ -120,16 +125,14 @@ static string get_ipmitool_path(void)
 }
 
 /* Get System Firmware FRU information on BMC based system */
-static string bmc_get_fw_fru_info(string ipmitool)
+static string bmc_get_fw_fru_info(string ipmitool, string interface)
 {
-	string cmd = ipmitool + " fru";
+	string cmd = ipmitool + interface;
 	string fruData, fwData;
 	size_t start, end;
 
-	if (HelperFunctions::execCmd(cmd.c_str(), fruData)) {
-		cout << "Failed to execute ipmitool command" << endl;
+	if (HelperFunctions::execCmd(cmd.c_str(), fruData))
 		return string();
-	}
 
 	start = fruData.find("System Firmware");
 	if (start == string::npos)
@@ -148,10 +151,140 @@ static string bmc_get_fw_fru_info(string ipmitool)
 	return fwData;
 
 parse_err:
-	cout << "Failed to get System Firmware information" << endl;
 	return string();
 }
 
+static string read_dt_property(const string& path, const string& attrName)
+{
+	struct stat info;
+	string fullPath;
+	string ret = "";
+
+	ostringstream os;
+	os << path << "/" << attrName;
+	fullPath = os.str( );
+
+	if (stat(fullPath.c_str( ), &info) != 0) {
+		ostringstream os;
+		if (errno != ENOENT) {
+			os << "Error statting " << fullPath << ", errno: " << errno;
+			Logger().log( os.str( ), LOG_ERR );
+		}
+		return ret;
+	}
+
+	ifstream attrIn;
+	attrIn.exceptions ( std::ifstream::failbit | std::ifstream::badbit );
+	try {
+		attrIn.open( fullPath.c_str( ) );
+	}
+	catch (std::ifstream::failure &e) {
+		ostringstream os;
+		os << "Error opening " << fullPath;
+		Logger().log(os.str( ), LOG_WARNING);
+		return ret;
+	}
+
+	if (attrIn) {
+		char * strBuf;
+		try
+		{
+			strBuf = new char [ info.st_size + 1 ];
+		}
+		catch (exception& e)
+		{
+			return ret;
+		}
+		memset( strBuf, '\0', info.st_size + 1 );
+		attrIn.read( strBuf, info.st_size );
+		ret = strBuf;
+		attrIn.close( );
+		delete [] strBuf;
+	}
+	return ret;
+}
+
+/* Get system firmware information on BMC based system via device tree */
+static string bmc_get_fw_dt_info(void)
+{
+	string fwdata, tag, val, prod_ver = "", prod_extra = "", prop_ver = "";
+	struct dirent *ent;
+	DIR * pDBdir = NULL;
+	/* Properties to ignore from DT/ibm,firmware-versions node */
+	const char *ignore_dt[] = {"phandle", "name"};
+	/* Properties to look for version string */
+	const char *version_prop[] = {"version", "IBM", "open-power", "buildroot"};
+	int i;
+	bool ignore_dt_flag = false;
+
+	pDBdir = opendir(FW_VERSION_DT_NODE);
+	if (pDBdir == NULL) {
+		stringstream os;
+		os << "Error opening directory " << FW_VERSION_DT_NODE << endl;
+		Logger().log(os.str( ), LOG_ERR);
+		return string("");
+	}
+
+	/* Get the version property */
+        for (i = 0; i < (int)(sizeof(version_prop)/sizeof(char *)); i++) {
+                string path = FW_VERSION_DT_NODE + string(version_prop[i]);
+                if (HelperFunctions::file_exists(path)) {
+                        prop_ver = version_prop[i];
+                        break;
+                }
+        }
+
+	fwdata = string("\n Product Name          : OpenPOWER Firmware\n");
+
+	/*
+	 * Different BMCs uses different device tree property to display
+	 * product name. On P9 system OPAL firmware takes care of parsing
+	 * and it will add version property under device tree. But on older
+	 * systems will have IBM or open-power or buildroot depending on
+	 * who builds firmware image.
+	 */
+	if (prop_ver != string("")) {
+		tag = string(" Product Version       : ");
+		prod_ver = read_dt_property(string(FW_VERSION_DT_NODE), prop_ver);
+		if (prod_ver != string("")) {
+			if (prop_ver.compare("version") == 0)
+				prod_ver = tag + prod_ver + string("\n");
+			else
+				prod_ver = tag + prop_ver + string("-") + prod_ver + string("\n");
+		}
+	}
+
+	/* Form the product extra items using remaining properties */
+	while ((ent = readdir( pDBdir )) != NULL) {
+		string fname = ent->d_name;
+		for (i = 0; i < (int)(sizeof(ignore_dt)/sizeof(char *)); i++) {
+			if (fname.compare(string(ignore_dt[i])) == 0) {
+				ignore_dt_flag = true;
+				break;
+			}
+		}
+
+		if (ignore_dt_flag == true) {
+			ignore_dt_flag = false;
+			continue;
+		}
+
+		if (prop_ver != string("")) {
+			if (fname.compare(prop_ver) == 0)
+				continue;
+		}
+
+		tag = string(" Product Extra         : \t");
+		val = read_dt_property(string(FW_VERSION_DT_NODE), fname);
+		if (val == string(""))
+			continue;
+		prod_extra = prod_extra + tag + fname + string("-") +  val + string("\n");
+	}
+
+	fwdata = fwdata + prod_ver + prod_extra;
+	return fwdata;
+}
+
 /* Get production version */
 static string bmc_get_product_version(string fwData)
 {
@@ -181,13 +314,30 @@ bool printSystem( const vector<Component*>& leaves )
 	 * based system. Hence we don't store this information in VPD db.
 	 */
 	if (PlatformCollector::isBMCBasedSystem()) {
-		string ipmitool = get_ipmitool_path();
-		if (ipmitool.empty())
-			return false;
+		string fwData;
+		if (!access(FW_VERSION_DT_NODE, F_OK | R_OK)) {
+			fwData = bmc_get_fw_dt_info();
+			if (fwData.empty())
+				return false;
+		} else {
+			string ipmitool = get_ipmitool_path();
+			if (ipmitool.empty())
+				return false;
 
-		string fwData = bmc_get_fw_fru_info(ipmitool);
-		if (fwData.empty())
-			return false;
+			/*
+			 * AMI BMC stack supports usb interface. If usb interface
+			 * is available use that, else fall back to inband interface.
+			 */
+			fwData = bmc_get_fw_fru_info(ipmitool, string(" -I usb fru"));
+			if (fwData.empty()) {
+				fwData = bmc_get_fw_fru_info(ipmitool, string(" fru"));
+				if (fwData.empty()) {
+					cerr << "Failed to get System Firmware \
+						information" << endl;
+					return false;
+				}
+			}
+		}
 
 		if ( all ) {
 			string pVersion = bmc_get_product_version(fwData);
@@ -401,8 +551,7 @@ void printVPD( System* root )
 	}
 
 	if( device == "" ) {
-		if (printSystem( root->getLeaves( ) ) != true)
-			return;
+		printSystem( root->getLeaves( ) );
 	}
 
 	if( !all && device == "" )
